@@ -25,9 +25,12 @@ from eventlet import greenthread
 from eventlet import queue
 from eventlet import timeout
 
+from oslo.vmware._i18n import _
+from oslo.vmware import constants
 from oslo.vmware import exceptions
-from oslo.vmware.openstack.common.gettextutils import _
+from oslo.vmware.objects import datastore as ds_obj
 from oslo.vmware import rw_handles
+from oslo.vmware import vim_util
 
 
 LOG = logging.getLogger(__name__)
@@ -74,27 +77,21 @@ class BlockingQueue(queue.LightQueue):
 
         :param data: data to be written
         """
-        LOG.debug("Writing %d data items into the queue.", len(data))
         self.put(data)
 
     # Below methods are provided in order to enable treating the queue
     # as a file handle.
-    # Note(vui): When file transfer size is not specified, we raise IOError to
-    # prevent incorrect predetermination of file length by readers.
 
     def seek(self, offset, whence=0):
-        if self._max_transfer_size is 0:
-            raise IOError(errno.ESPIPE, "Illegal seek")
+        """Set the file's current position at the offset.
+
+        This method throws IOError since seek cannot be supported for a pipe.
+        """
+        raise IOError(errno.ESPIPE, "Illegal seek")
 
     def tell(self):
-        """Get size of the file to be read.
-
-        We interpret _max_transfer_size=0 as stream mode and raise IOError
-        to prevent incorrect predetermination of file length by readers.
-        """
-        if self._max_transfer_size is 0:
-            raise IOError(errno.ESPIPE, "Illegal seek")
-        return self._max_transfer_size
+        """Get the current file position."""
+        return self._transferred
 
     def close(self):
         pass
@@ -397,6 +394,41 @@ def _start_transfer(context, timeout_secs, read_file_handle, max_data_size,
             write_file_handle.close()
 
 
+def download_image(image, image_meta, session, datastore, rel_path,
+                   bypass=True, timeout_secs=7200):
+    """Transfer an image to a datastore.
+
+    :param image: file-like iterator
+    :param image_meta: image metadata
+    :param session: VMwareAPISession object
+    :param datastore: Datastore object
+    :param rel_path: path where the file will be stored in the datastore
+    :param bypass: if set to True, bypass vCenter to download the image
+    :param timeout_secs: time in seconds to wait for the xfer to complete
+    """
+    image_size = int(image_meta['size'])
+    method = 'PUT'
+    if bypass:
+        hosts = datastore.get_connected_hosts(session)
+        host = ds_obj.Datastore.choose_host(hosts)
+        host_name = session.invoke_api(vim_util, 'get_object_property',
+                                       session.vim, host, 'name')
+        ds_url = datastore.build_url(session._scheme, host_name, rel_path,
+                                     constants.ESX_DATACENTER_PATH)
+        cookie = ds_url.get_transfer_ticket(session, method)
+        conn = ds_url.connect(method, image_size, cookie)
+    else:
+        ds_url = datastore.build_url(session._scheme, session._host, rel_path)
+        cookie = '%s=%s' % (constants.SOAP_COOKIE_KEY,
+                            session.vim.get_http_cookie().strip("\""))
+        conn = ds_url.connect(method, image_size, cookie)
+        conn.write = conn.send
+
+    read_handle = rw_handles.ImageReadHandle(image)
+    _start_transfer(None, timeout_secs, read_handle, image_size,
+                    write_file_handle=conn)
+
+
 def download_flat_image(context, timeout_secs, image_service, image_id,
                         **kwargs):
     """Download flat image from the image service to VMware server.
@@ -495,6 +527,35 @@ def download_stream_optimized_image(context, timeout_secs, image_service,
     return imported_vm
 
 
+def copy_stream_optimized_disk(
+        context, timeout_secs, write_handle, **kwargs):
+    """Copy virtual disk from VMware server to the given write handle.
+
+    :param context: context
+    :param timeout_secs: time in seconds to wait for the copy to complete
+    :param write_handle: copy destination
+    :param kwargs: keyword arguments to configure the source
+                   VMDK read handle
+    :raises: VimException, VimFaultException, VimAttributeException,
+             VimSessionOverLoadException, VimConnectionException,
+             ImageTransferException, ValueError
+    """
+    vmdk_file_path = kwargs.get('vmdk_file_path')
+    LOG.debug("Copying virtual disk: %(vmdk_path)s to %(dest)s.",
+              {'vmdk_path': vmdk_file_path,
+               'dest': write_handle.name})
+    file_size = kwargs.get('vmdk_size')
+    read_handle = rw_handles.VmdkReadHandle(kwargs.get('session'),
+                                            kwargs.get('host'),
+                                            kwargs.get('port'),
+                                            kwargs.get('vm'),
+                                            kwargs.get('vmdk_file_path'),
+                                            file_size)
+    _start_transfer(context, timeout_secs, read_handle, file_size,
+                    write_file_handle=write_handle)
+    LOG.debug("Downloaded virtual disk: %s.", vmdk_file_path)
+
+
 def upload_image(context, timeout_secs, image_service, image_id, owner_id,
                  **kwargs):
     """Upload the VM's disk file to image service.
