@@ -27,7 +27,7 @@ import urlparse
 
 import suds.sax.element as element
 
-from oslo.vmware.openstack.common.gettextutils import _LW
+from oslo.vmware._i18n import _LW
 from oslo.vmware import service
 from oslo.vmware import vim_util
 
@@ -41,17 +41,21 @@ class Pbm(service.Service):
     """Service class that provides access to the Storage Policy API."""
 
     def __init__(self, protocol='https', host='localhost', port=443,
-                 wsdl_url=None):
+                 wsdl_url=None, cacert=None, insecure=True):
         """Constructs a PBM service client object.
 
         :param protocol: http or https
         :param host: server IP address or host name
         :param port: port for connection
         :param wsdl_url: PBM WSDL url
+        :param cacert: Specify a CA bundle file to use in verifying a
+                       TLS (https) server certificate.
+        :param insecure: Verify HTTPS connections using system certificates,
+                         used only if cacert is not specified
         """
         base_url = service.Service.build_base_url(protocol, host, port)
         soap_url = base_url + '/pbm'
-        super(Pbm, self).__init__(wsdl_url, soap_url)
+        super(Pbm, self).__init__(wsdl_url, soap_url, cacert, insecure)
 
     def set_soap_cookie(self, cookie):
         """Set the specified vCenter session cookie in the SOAP header
