@@ -17,9 +17,10 @@
 The VMware API utility module.
 """
 
-import netaddr
 import suds
 
+from oslo.utils import timeutils
+
 
 def get_moref(value, type_):
     """Get managed object reference.
@@ -87,6 +88,12 @@ def build_recursive_traversal_spec(client_factory):
                                      'vmFolder',
                                      False,
                                      [visit_folders_select_spec])
+    dc_to_netf = build_traversal_spec(client_factory,
+                                      'dc_to_netf',
+                                      'Datacenter',
+                                      'networkFolder',
+                                      False,
+                                      [visit_folders_select_spec])
 
     # Next hop from HostSystem
     h_to_vm = build_traversal_spec(client_factory,
@@ -168,6 +175,7 @@ def build_recursive_traversal_spec(client_factory):
                                            h_to_vm,
                                            dc_to_hf,
                                            dc_to_vmf,
+                                           dc_to_netf,
                                            cr_to_ds,
                                            cr_to_h,
                                            cr_to_rp,
@@ -368,14 +376,111 @@ def get_object_property(vim, moref, property_name):
     return prop_val
 
 
-def get_soap_url(protocol, host, path='sdk'):
-    """Return ESX/VC server's SOAP service URL.
+def find_extension(vim, key):
+    """Looks for an existing extension.
 
-    :param protocol: https or http
-    :param host: server IP address[:port] or host name[:port]
-    :param path: path part of the SOAP URL
-    :returns: SOAP service URL
+    :param vim: Vim object
+    :param key: the key to search for
+    :returns: the data object Extension or None
     """
-    if netaddr.valid_ipv6(host):
-        return '%s://[%s]/%s' % (protocol, host, path)
-    return '%s://%s/%s' % (protocol, host, path)
+    extension_manager = vim.service_content.extensionManager
+    return vim.client.service.FindExtension(extension_manager, key)
+
+
+def register_extension(vim, key, type, label='OpenStack',
+                       summary='OpenStack services', version='1.0'):
+    """Create a new extention.
+
+    :param vim: Vim object
+    :param key: the key for the extension
+    :param type: Managed entity type, as defined by the extension. This
+                 matches the type field in the configuration about a
+                 virtual machine or vApp
+    :param label: Display label
+    :param summary: Summary description
+    :param version: Extension version number as a dot-separated string
+    """
+    extension_manager = vim.service_content.extensionManager
+    client_factory = vim.client.factory
+    os_ext = client_factory.create('ns0:Extension')
+    os_ext.key = key
+    entity_info = client_factory.create('ns0:ExtManagedEntityInfo')
+    entity_info.type = type
+    os_ext.managedEntityInfo = [entity_info]
+    os_ext.version = version
+    desc = client_factory.create('ns0:Description')
+    desc.label = label
+    desc.summary = summary
+    os_ext.description = desc
+    os_ext.lastHeartbeatTime = timeutils.strtime()
+    vim.client.service.RegisterExtension(extension_manager, os_ext)
+
+
+def get_vc_version(session):
+    """Return the dot-separated vCenter version string. For example, "1.2".
+
+    :param session: vCenter soap session
+    :return: vCenter version
+    """
+    return session.vim.service_content.about.version
+
+
+def get_inventory_path(vim, entity_ref, max_objects=100):
+    """Get the inventory path of a managed entity.
+
+    :param vim: Vim object
+    :param entity_ref: managed entity reference
+    :param max_objects: maximum number of objects that should be returned in
+                        a single call
+    :return: inventory path of the entity_ref
+    """
+    client_factory = vim.client.factory
+    property_collector = vim.service_content.propertyCollector
+
+    prop_spec = build_property_spec(client_factory, 'ManagedEntity',
+                                    ['name', 'parent'])
+    select_set = build_selection_spec(client_factory, 'ParentTraversalSpec')
+    select_set = build_traversal_spec(
+        client_factory, 'ParentTraversalSpec', 'ManagedEntity', 'parent',
+        False, [select_set])
+    obj_spec = build_object_spec(client_factory, entity_ref, select_set)
+    prop_filter_spec = build_property_filter_spec(client_factory,
+                                                  [prop_spec], [obj_spec])
+    options = client_factory.create('ns0:RetrieveOptions')
+    options.maxObjects = max_objects
+    retrieve_result = vim.RetrievePropertiesEx(
+        property_collector,
+        specSet=[prop_filter_spec],
+        options=options)
+    entity_name = None
+    propSet = None
+    path = ""
+    while retrieve_result:
+        for obj in retrieve_result.objects:
+            if hasattr(obj, 'propSet'):
+                propSet = obj.propSet
+                if len(propSet) >= 1 and not entity_name:
+                    entity_name = propSet[0].val
+                elif len(propSet) >= 1:
+                    path = '%s/%s' % (propSet[0].val, path)
+        retrieve_result = continue_retrieval(vim, retrieve_result)
+    # NOTE(arnaud): slice to exclude the root folder from the result.
+    if propSet is not None and len(propSet) > 0:
+        path = path[len(propSet[0].val):]
+    if entity_name is None:
+        entity_name = ""
+    return '%s%s' % (path, entity_name)
+
+
+def get_http_service_request_spec(client_factory, method, uri):
+    """Build a HTTP service request spec.
+
+    :param client_factory: factory to get API input specs
+    :param method: HTTP method (GET, POST, PUT)
+    :param uri: target URL
+    """
+    http_service_request_spec = client_factory.create(
+        'ns0:SessionManagerHttpServiceRequestSpec')
+    http_service_request_spec.method = method
+    http_service_request_spec.url = uri
+    return http_service_request_spec
