@@ -1,3 +1,4 @@
+# coding=utf-8
 # Copyright (c) 2014 VMware, Inc.
 # All Rights Reserved.
 #
@@ -19,6 +20,7 @@ Unit tests for session management and API invocation classes.
 
 from eventlet import greenthread
 import mock
+import suds
 
 from oslo.vmware import api
 from oslo.vmware import exceptions
@@ -106,6 +108,7 @@ class VMwareAPISessionTest(base.TestCase):
         self.addCleanup(patcher.stop)
         self.VimMock = patcher.start()
         self.VimMock.side_effect = lambda *args, **kw: mock.MagicMock()
+        self.cert_mock = mock.Mock()
 
     def _create_api_session(self, _create_session, retry_count=10,
                             task_poll_interval=1):
@@ -116,7 +119,9 @@ class VMwareAPISessionTest(base.TestCase):
                                     task_poll_interval,
                                     'https',
                                     _create_session,
-                                    port=VMwareAPISessionTest.PORT)
+                                    port=VMwareAPISessionTest.PORT,
+                                    cacert=self.cert_mock,
+                                    insecure=False)
 
     def test_vim(self):
         api_session = self._create_api_session(False)
@@ -124,7 +129,9 @@ class VMwareAPISessionTest(base.TestCase):
         self.VimMock.assert_called_with(protocol=api_session._scheme,
                                         host=VMwareAPISessionTest.SERVER_IP,
                                         port=VMwareAPISessionTest.PORT,
-                                        wsdl_url=api_session._vim_wsdl_loc)
+                                        wsdl_url=api_session._vim_wsdl_loc,
+                                        cacert=self.cert_mock,
+                                        insecure=False)
 
     @mock.patch.object(pbm, 'Pbm')
     def test_pbm(self, pbm_mock):
@@ -257,6 +264,27 @@ class VMwareAPISessionTest(base.TestCase):
                           module,
                           'api')
 
+    def test_invoke_api_with_vim_fault_exception_details(self):
+        api_session = self._create_api_session(True)
+        fault_string = 'Invalid property.'
+        fault_list = [exceptions.INVALID_PROPERTY]
+        details = {u'name': suds.sax.text.Text(u'фира')}
+
+        module = mock.Mock()
+        module.api.side_effect = exceptions.VimFaultException(fault_list,
+                                                              fault_string,
+                                                              details=details)
+        e = self.assertRaises(exceptions.InvalidPropertyException,
+                              api_session.invoke_api,
+                              module,
+                              'api')
+        details_str = u"{'name': 'фира'}"
+        expected_str = "%s\nFaults: %s\nDetails: %s" % (fault_string,
+                                                        fault_list,
+                                                        details_str)
+        self.assertEqual(expected_str, unicode(e))
+        self.assertEqual(details, e.details)
+
     def test_invoke_api_with_empty_response(self):
         api_session = self._create_api_session(True)
         vim_obj = api_session.vim
@@ -465,3 +493,32 @@ class VMwareAPISessionTest(base.TestCase):
 
         for k, v in _unknown_exceptions.iteritems():
             self._poll_task_well_known_exceptions(k, v)
+
+    def _create_subclass_exception(self):
+        class VimSubClass(exceptions.VMwareDriverException):
+            pass
+        return VimSubClass
+
+    def test_register_fault_class(self):
+        exc = self._create_subclass_exception()
+        exceptions.register_fault_class('ValueError', exc)
+        self.assertEqual(exc, exceptions.get_fault_class('ValueError'))
+
+    def test_register_fault_class_override(self):
+        exc = self._create_subclass_exception()
+        exceptions.register_fault_class(exceptions.ALREADY_EXISTS, exc)
+        self.assertEqual(exc,
+                         exceptions.get_fault_class(exceptions.ALREADY_EXISTS))
+
+    def test_register_fault_classi_invalid(self):
+        self.assertRaises(TypeError,
+                          exceptions.register_fault_class,
+                          'ValueError', ValueError)
+
+    def test_update_pbm_wsdl_loc(self):
+        session = mock.Mock()
+        session.key = "12345"
+        api_session = self._create_api_session(False)
+        self.assertIsNone(api_session._pbm_wsdl_loc)
+        api_session.pbm_wsdl_loc_set('fake_wsdl')
+        self.assertEqual('fake_wsdl', api_session._pbm_wsdl_loc)
